// Generated by CoffeeScript 1.6.1
(function() {
  var dataPath, fs, path, resourcesPath, rimraf, testDoccoRun, testPath;

  path = require('path');

  fs = require('fs');

  rimraf = require('rimraf');

  testPath = path.dirname(fs.realpathSync(__filename));

  dataPath = path.join(testPath, "data");

  resourcesPath = path.normalize(path.join(testPath, "/../resources"));

  testDoccoRun = function(testName, sources, options, callback) {
    var cleanup, destPath;
    if (options == null) {
      options = null;
    }
    if (callback == null) {
      callback = null;
    }
    destPath = path.join(dataPath, testName);
    cleanup = function(callback) {
      return rimraf(destPath, callback);
    };
    return cleanup(function(error) {
      equal(!error, true, "path cleaned up properly");
      if (options != null) {
        options.output = destPath;
      }
      return Docco.document(sources, options, function() {
        var expected, files, found, src, _i, _len;
        files = [];
        for (_i = 0, _len = sources.length; _i < _len; _i++) {
          src = sources[_i];
          files = files.concat(Docco.resolveSource(src));
        }
        expected = files.length + 1;
        found = fs.readdirSync(destPath).length;
        equal(found, expected, "find expected output (" + expected + " files) - (" + found + ")");
        if (callback != null) {
          return callback();
        }
      });
    });
  };

  test("custom JST template file", function() {
    return testDoccoRun("custom_jst", ["" + testPath + "/*.coffee"], {
      template: "" + resourcesPath + "/pagelet.jst"
    });
  });

  test("custom CSS file", function() {
    return testDoccoRun("custom_css", ["" + testPath + "/*.coffee"], {
      css: "" + resourcesPath + "/pagelet.css"
    });
  });

  test("specify an extension", function() {
    return testDoccoRun("specify_extension", ["" + testPath + "/comments/noextension"], {
      extension: ".coffee"
    });
  });

  test("single line comment parsing", function() {
    var commentsPath, ext, l, languageKeys, options, testNextLanguage;
    commentsPath = path.join(testPath, "comments");
    options = {
      template: "" + commentsPath + "/comments.jst"
    };
    languageKeys = (function() {
      var _ref, _results;
      _ref = Docco.languages;
      _results = [];
      for (ext in _ref) {
        l = _ref[ext];
        _results.push(ext);
      }
      return _results;
    })();
    testNextLanguage = function(keys, callback) {
      var extension, language, languageExample, languageOutput, languagePath, languageTest;
      if (!keys.length) {
        return typeof callback === "function" ? callback() : void 0;
      }
      extension = keys.shift();
      language = Docco.languages[extension];
      languageExample = path.join(commentsPath, "" + language.name + extension);
      languageTest = "comments_test/" + language.name;
      languagePath = path.join(dataPath, languageTest);
      languageOutput = path.join(languagePath, "" + language.name + ".html");
      if (!path.existsSync(languageExample)) {
        return testNextLanguage(keys, callback);
      }
      return testDoccoRun(languageTest, [languageExample], options, function() {
        var c, comments, content, expected;
        equal(true, path.existsSync(languageOutput), "" + languageOutput + " -> output file created properly");
        content = fs.readFileSync(languageOutput).toString();
        comments = (function() {
          var _i, _len, _ref, _results;
          _ref = content.split(',');
          _results = [];
          for (_i = 0, _len = _ref.length; _i < _len; _i++) {
            c = _ref[_i];
            if (c.trim() !== '') {
              _results.push(c.trim());
            }
          }
          return _results;
        })();
        equal(true, comments.length >= 1, 'expect at least the descriptor comment');
        expected = parseInt(comments[0]);
        equal(comments.length, expected, ["", "" + (path.basename(languageOutput)) + " comments", "------------------------", " expected : " + expected, " found    : " + comments.length].join('\n'));
        return testNextLanguage(keys, callback);
      });
    };
    return testNextLanguage(languageKeys.slice());
  });

  test("url references", function() {
    return Docco.ensureDirectory(dataPath, function() {
      var outFile, outPath, sourceFile;
      sourceFile = "" + dataPath + "/_urlref.coffee";
      fs.writeFileSync(sourceFile, ["# Look at this link to [Google][]!", "console.log 'This must be Thursday.'", "# And this link to [Google][] as well.", "console.log 'I never could get the hang of Thursdays.'", "# [google]: http://www.google.com"].join('\n'));
      outPath = path.join(dataPath, "_urlreferences");
      outFile = "" + outPath + "/_urlref.html";
      return rimraf(outPath, function(error) {
        equal(!error, true);
        return Docco.document([sourceFile], {
          output: outPath
        }, function() {
          var contents, count;
          contents = fs.readFileSync(outFile).toString();
          count = contents.match(/<a\shref="http:\/\/www.google.com">Google<\/a>/g);
          return equal(count.length, 2, "find expected (2) resolved url references");
        });
      });
    });
  });

  test("create complex paths that do not exist", function() {
    var exist, outputPath;
    exist = fs.existsSync || path.existsSync;
    outputPath = path.join(dataPath, 'complex/path/that/doesnt/exist');
    return rimraf(outputPath, function(error) {
      equal(!error, true);
      return Docco.ensureDirectory(outputPath, function() {
        var stat;
        equal(exist(outputPath), true, 'created output path');
        stat = fs.statSync(outputPath);
        return equal(stat.isDirectory(), true, "target is directory");
      });
    });
  });

}).call(this);
